@@ -1,13 +1,10 @@
-
-require 5;
-# Time-stamp: "2003-09-15 00:32:45 ADT"
 package HTML::TreeBuilder;
 
 use strict;
 use integer; # vroom vroom!
 use Carp ();
 use vars qw(@ISA $VERSION $DEBUG);
-$VERSION = '3.13';
+$VERSION = '3.23';
 
 #---------------------------------------------------------------------------
 # Make a 'DEBUG' constant...
@@ -154,8 +151,8 @@ sub new { # constructor!
   $self->{'_warn'}               = 0;
   $self->{'_no_space_compacting'}= 0;
   $self->{'_store_comments'}     = 0;
+  $self->{'_store_declarations'} = 1;
   $self->{'_store_pis'}          = 0;
-  $self->{'_store_declarations'} = 0;
   $self->{'_p_strict'} = 0;
   
   # Parse attributes passed in as arguments
@@ -690,7 +687,7 @@ sub warning {
          ($sibs = ( $par = $self->{'_pos'} || $self )->{'_content'})
          and @$sibs  # parent already has content
          and !ref($sibs->[-1])  # and the last one there is a text node
-         and $sibs->[-1] !~ m<\S>s  # and it's all whitespace
+         and $sibs->[-1] !~ m<[^\n\r\f\t ]>s  # and it's all whitespace
 
          and (  # one of these has to be eligible...
                $HTML::TreeBuilder::canTighten{$tag}
@@ -1024,8 +1021,8 @@ sub warning {
         $pos->push_content($text);
     } else {
         # return unless $text =~ /\S/;  # This is sometimes wrong
-        
-        if (!$self->{'_implicit_tags'} || $text !~ /\S/) {
+
+        if (!$self->{'_implicit_tags'} || $text !~ /[^\n\r\f\t ]/) {
             # don't change anything
         } elsif ($ptag eq 'head' or $ptag eq 'noframes') {
             if($self->{'_implicit_body_p_tag'}) {
@@ -1103,8 +1100,9 @@ sub warning {
         #print "POS is now $pos, ", $pos->{'_tag'}, "\n";
         
         return if $ignore_text;
-        $text =~ s/\s+/ /g unless $no_space_compacting ;  # canonical space
-        
+        $text =~ s/[\n\r\f\t ]+/ /g  # canonical space
+            unless $no_space_compacting ;
+
         print
           $indent, " (Attaching text node ($nugget) under ",
           # was: $self->{'_pos'} ? $self->{'_pos'}{'_tag'} : $self->{'_tag'},
@@ -1167,24 +1165,17 @@ sub comment {
   return $e;
 }
 
-#==========================================================================
-# TODO: currently this puts declarations in just the wrong place.
-#  How to correct? look at pos->_content, and go to insert at end, 
-#  but back up before any head elements?  Do that just if implicit
-#  mode is on?
-
 sub declaration {
   return if $_[0]{'_stunted'};
   # Accept a "here's a markup declaration" signal from HTML::Parser.
 
-  return unless $_[0]->{'_store_declarations'};
   my($self, $text) = @_;
   my $pos = $self->{'_pos'} || $self;
-  
+
   if(DEBUG) {
     my @lineage_tags = $pos->lineage_tag_names;
     my $indent = '  ' x (1 + @lineage_tags);
-    
+
     my $nugget = (length($text) <= 25) ? $text : (substr($text,0,25) . '...');
     $nugget =~ s<([\x00-\x1F])>
                  <'\\x'.(unpack("H2",$1))>eg;
@@ -1196,14 +1187,8 @@ sub declaration {
   (my $e = (
     $self->{'_element_class'} || 'HTML::Element'
    )->new('~declaration'))->{'text'} = $text;
-  $pos->push_content($e);
-  ++($self->{'_element_count'});
 
-  &{  $self->{'_tweak_~declaration'}
-      || $self->{'_tweak_*'}
-      || return $e
-   }(map $_, $e,   '~declaration', $self);
-  
+  $self->{_decl} = $e;
   return $e;
 }
 
@@ -1612,7 +1597,7 @@ tree for a file consisting of just this:
   <li>I like pie!
 
 Then you would get that with C<< @nodes = $root->guts(); >>.
-It so happens that in this case, @notes will contain just one
+It so happens that in this case, C<@nodes> will contain just one
 element object, representing the "li" node (with "I like pie!" being
 its text child node).  However, consider if you were parsing this:
 
@@ -1741,12 +1726,8 @@ while parsing content into C<$root>.  Currently, this is off by default.
 =item $root->store_declarations(value)
 
 This determines whether TreeBuilder will normally store markup
-declarations found while parsing content into C<$root>.  Currently,
-this is off by default.
-
-It is somewhat of a known bug (to be fixed one of these days, if
-anyone needs it?) that declarations in the preamble (before the "html"
-start-tag) end up actually I<under> the "html" element.
+declarations found while parsing content into C<$root>.  This is on
+by default.
 
 =item $root->store_pis(value)
 
@@ -1853,7 +1834,9 @@ When a document parses in a way different from how you think it
 should, I ask that you report this to me as a bug.  The first thing
 you should do is copy the document, trim out as much of it as you can
 while still producing the bug in question, and I<then> email me that
-mini-document I<and> the code you're using to parse it, at C<sburke@cpan.org>.
+mini-document I<and> the code you're using to parse it, to the HTML::Tree
+bug queue at C<bug-html-tree at rt.cpan.org>.
+
 Include a note as to how it 
 parses (presumably including its $tree->dump output), and then a
 I<careful and clear> explanation of where you think the parser is
@@ -1867,7 +1850,9 @@ L<HTML::DOMbo>
 
 =head1 COPYRIGHT
 
-Copyright 1995-1998 Gisle Aas; copyright 1999-2002 Sean M. Burke.
+Copyright 1995-1998 Gisle Aas, 1999-2004 Sean M. Burke, 2005 Andy Lester,
+2006 Pete Krawczyk.
+
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
@@ -1877,43 +1862,8 @@ merchantability or fitness for a particular purpose.
 
 =head1 AUTHOR
 
-Original author Gisle Aas E<lt>gisle@aas.noE<gt>; current maintainer
-Sean M. Burke, E<lt>sburke@cpan.orgE<gt>
+Currently maintained by Pete Krawczyk C<< <petek@cpan.org> >>
+
+Original authors: Gisle Aas, Sean Burke and Andy Lester.
 
 =cut
-
-
-CUCUMBER AND ORANGE SALAD
-
-Adapted from:
-
-Holzner, Yupa.  /Great Thai Cooking for My American Friends: Creative
-Thai Dishes Made Easy/.  Royal House 1989.  ISBN:0930440277
-
-Makes about four servings.
-
- 1 small greenhouse cucumber, sliced thin.
- 2 navel oranges, peeled, separated into segments (seeded if
-   necessary), and probably chopped into chunks.
-
-Dressing:
- 3 tablespoons rice vinegar
- .5 teaspoon salt
- 1 tablespoon sugar
- 1 teaspoon roasted sesame seeds
- .5 tablespoons mirin
-
-Mix all ingredients in the dressing.  Pour the dressing over the
-oranges and cucumbers, and toss.  Serve.
-
-
-All ingredient quantities are approximate.  Adjust to taste.  Multipy
-quantities for more servings.
-
-Note: the sesame seeds you get in an American supermarket are almost
-certain to be unroasted, suspiciously waxy-looking, and astronomically
-overpriced.  Good cheap sesame seeds are available at most Asian
-markets -- they usually come roasted; if they're unroasted, roast them
-in a skillet for a few short minutes as needed.
-
-[End Recipe]
